@@ -7,23 +7,26 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include "ChildSession.hpp"
 
 #include <sstream>
 
+#define LOK_USE_UNSTABLE_API
+#include <LibreOfficeKit/LibreOfficeKitEnums.h>
+
 #include <Poco/JSON/Object.h>
 #include <Poco/JSON/Parser.h>
 #include <Poco/Net/WebSocket.h>
 #include <Poco/StringTokenizer.h>
 #include <Poco/URI.h>
 
-#include "common/FileUtil.hpp"
+#include <common/FileUtil.hpp>
 #include "KitHelper.hpp"
-#include "Log.hpp"
-#include "Png.hpp"
-#include "Util.hpp"
+#include <Log.hpp>
+#include <Png.hpp>
+#include <Util.hpp>
 
 using Poco::JSON::Object;
 using Poco::JSON::Parser;
@@ -77,7 +80,7 @@ bool ChildSession::_handleInput(const char *buffer, int length)
 {
     LOG_TRC(getName() << ": handling [" << getAbbreviatedMessage(buffer, length) << "].");
     const std::string firstLine = getFirstLine(buffer, length);
-    const auto tokens = LOOLProtocol::tokenize(firstLine.data(), firstLine.size());
+    const std::vector<std::string> tokens = LOOLProtocol::tokenize(firstLine.data(), firstLine.size());
 
     if (LOOLProtocol::tokenIndicatesUserInteraction(tokens[0]))
     {
@@ -202,6 +205,10 @@ bool ChildSession::_handleInput(const char *buffer, int length)
     {
         return getStatus(buffer, length);
     }
+    else if (tokens[0] == "paintwindow")
+    {
+        return renderWindow(buffer, length, tokens);
+    }
     else if (tokens[0] == "tile" || tokens[0] == "tilecombine")
     {
         assert(false && "Tile traffic should go through the DocumentBroker-LoKit WS.");
@@ -213,20 +220,25 @@ bool ChildSession::_handleInput(const char *buffer, int length)
 
         assert(tokens[0] == "clientzoom" ||
                tokens[0] == "clientvisiblearea" ||
+               tokens[0] == "outlinestate" ||
                tokens[0] == "downloadas" ||
                tokens[0] == "getchildid" ||
                tokens[0] == "gettextselection" ||
                tokens[0] == "paste" ||
                tokens[0] == "insertfile" ||
                tokens[0] == "key" ||
+               tokens[0] == "textinput" ||
+               tokens[0] == "windowkey" ||
                tokens[0] == "mouse" ||
+               tokens[0] == "windowmouse" ||
                tokens[0] == "uno" ||
                tokens[0] == "selecttext" ||
                tokens[0] == "selectgraphic" ||
                tokens[0] == "resetselection" ||
                tokens[0] == "saveas" ||
                tokens[0] == "useractive" ||
-               tokens[0] == "userinactive");
+               tokens[0] == "userinactive" ||
+               tokens[0] == "windowcommand");
 
         if (tokens[0] == "clientzoom")
         {
@@ -236,6 +248,10 @@ bool ChildSession::_handleInput(const char *buffer, int length)
         {
             return clientVisibleArea(buffer, length, tokens);
         }
+        else if (tokens[0] == "outlinestate")
+        {
+            return outlineState(buffer, length, tokens);
+        }
         else if (tokens[0] == "downloadas")
         {
             return downloadAs(buffer, length, tokens);
@@ -258,11 +274,23 @@ bool ChildSession::_handleInput(const char *buffer, int length)
         }
         else if (tokens[0] == "key")
         {
-            return keyEvent(buffer, length, tokens);
+            return keyEvent(buffer, length, tokens, LokEventTargetEnum::Document);
+        }
+        else if (tokens[0] == "textinput")
+        {
+            return extTextInputEvent(buffer, length, tokens);
+        }
+        else if (tokens[0] == "windowkey")
+        {
+            return keyEvent(buffer, length, tokens, LokEventTargetEnum::Window);
         }
         else if (tokens[0] == "mouse")
         {
-            return mouseEvent(buffer, length, tokens);
+            return mouseEvent(buffer, length, tokens, LokEventTargetEnum::Document);
+        }
+        else if (tokens[0] == "windowmouse")
+        {
+            return mouseEvent(buffer, length, tokens, LokEventTargetEnum::Window);
         }
         else if (tokens[0] == "uno")
         {
@@ -292,6 +320,10 @@ bool ChildSession::_handleInput(const char *buffer, int length)
         {
             setIsActive(false);
         }
+        else if (tokens[0] == "windowcommand")
+        {
+            sendWindowCommand(buffer, length, tokens);
+        }
         else
         {
             assert(false && "Unknown command token.");
@@ -352,7 +384,7 @@ bool ChildSession::loadDocument(const char * /*buffer*/, int /*length*/, const s
 
     // Respond by the document status
     LOG_DBG("Sending status after loading view " << _viewId << ".");
-    const auto status = LOKitHelper::documentStatus(getLOKitDocument()->get());
+    const std::string status = LOKitHelper::documentStatus(getLOKitDocument()->get());
     if (status.empty() || !sendTextFrame("status: " + status))
     {
         LOG_ERR("Failed to get/forward document status [" << status << "].");
@@ -420,7 +452,9 @@ bool ChildSession::sendFontRendering(const char* /*buffer*/, int /*length*/, con
         return sendTextFrame(output.data(), output.size());
     }
 
-    if (Png::encodeBufferToPNG(ptrFont, width, height, output, LOK_TILEMODE_RGBA))
+    const auto mode = static_cast<LibreOfficeKitTileMode>(getLOKitDocument()->getTileMode());
+
+    if (Png::encodeBufferToPNG(ptrFont, width, height, output, mode))
     {
         bSuccess = sendTextFrame(output.data(), output.size());
     }
@@ -460,17 +494,17 @@ namespace
 void insertUserNames(const std::map<int, UserInfo>& viewInfo, std::string& json)
 {
     Poco::JSON::Parser parser;
-    auto root = parser.parse(json).extract<Poco::JSON::Object::Ptr>();
+    const Poco::JSON::Object::Ptr root = parser.parse(json).extract<Poco::JSON::Object::Ptr>();
     std::vector<std::string> directions { "Undo", "Redo" };
     for (auto& directionName : directions)
     {
-        auto direction = root->get(directionName).extract<Poco::JSON::Object::Ptr>();
+        Poco::JSON::Object::Ptr direction = root->get(directionName).extract<Poco::JSON::Object::Ptr>();
         if (direction->get("actions").type() == typeid(Poco::JSON::Array::Ptr))
         {
-            auto actions = direction->get("actions").extract<Poco::JSON::Array::Ptr>();
+            Poco::JSON::Array::Ptr actions = direction->get("actions").extract<Poco::JSON::Array::Ptr>();
             for (auto& actionVar : *actions)
             {
-                auto action = actionVar.extract<Poco::JSON::Object::Ptr>();
+                Poco::JSON::Object::Ptr action = actionVar.extract<Poco::JSON::Object::Ptr>();
                 int viewId = action->getValue<int>("viewId");
                 auto it = viewInfo.find(viewId);
                 if (it != viewInfo.end())
@@ -519,7 +553,7 @@ bool ChildSession::getCommandValues(const char* /*buffer*/, int /*length*/, cons
     else
     {
         values = getLOKitDocument()->getCommandValues(command.c_str());
-        success = sendTextFrame("commandvalues: " + std::string(values == nullptr ? "" : values));
+        success = sendTextFrame("commandvalues: " + std::string(values == nullptr ? "{}" : values));
         std::free(values);
     }
 
@@ -573,6 +607,34 @@ bool ChildSession::clientVisibleArea(const char* /*buffer*/, int /*length*/, con
     return true;
 }
 
+bool ChildSession::outlineState(const char* /*buffer*/, int /*length*/, const std::vector<std::string>& tokens)
+{
+    std::string type, state;
+    int level, index;
+
+    if (tokens.size() != 5 ||
+        !getTokenString(tokens[1], "type", type) ||
+        (type != "column" && type != "row") ||
+        !getTokenInteger(tokens[2], "level", level) ||
+        !getTokenInteger(tokens[3], "index", index) ||
+        !getTokenString(tokens[4], "state", state) ||
+        (state != "visible" && state != "hidden"))
+    {
+        sendTextFrame("error: cmd=outlinestate kind=syntax");
+        return false;
+    }
+
+    bool column = type == "column";
+    bool hidden = state == "hidden";
+
+    std::unique_lock<std::mutex> lock(_docManager.getDocumentMutex());
+
+    getLOKitDocument()->setView(_viewId);
+
+    getLOKitDocument()->setOutlineState(column, level, index, hidden);
+    return true;
+}
+
 bool ChildSession::downloadAs(const char* /*buffer*/, int /*length*/, const std::vector<std::string>& tokens)
 {
     std::string name, id, format, filterOptions;
@@ -596,7 +658,7 @@ bool ChildSession::downloadAs(const char* /*buffer*/, int /*length*/, const std:
     }
 
     // The file is removed upon downloading.
-    const auto tmpDir = FileUtil::createRandomDir(JAILED_DOCUMENT_ROOT);
+    const std::string tmpDir = FileUtil::createRandomDir(JAILED_DOCUMENT_ROOT);
     // Prevent user inputting anything funny here.
     // A "name" should always be a name, not a path
     const Poco::Path filenameParam(name);
@@ -706,23 +768,68 @@ bool ChildSession::insertFile(const char* /*buffer*/, int /*length*/, const std:
     return true;
 }
 
-bool ChildSession::keyEvent(const char* /*buffer*/, int /*length*/, const std::vector<std::string>& tokens)
+bool ChildSession::extTextInputEvent(const char* /*buffer*/, int /*length*/,
+                                     const std::vector<std::string>& tokens)
+{
+    int id, type;
+    std::string text;
+    if (tokens.size() < 4 ||
+        !getTokenInteger(tokens[1], "id", id) || id < 0 ||
+        !getTokenKeyword(tokens[2], "type",
+                        {{"input", LOK_EXT_TEXTINPUT}, {"end", LOK_EXT_TEXTINPUT_END}},
+                         type) ||
+        !getTokenString(tokens[3], "text", text))
+
+    {
+        sendTextFrame("error: cmd=" + std::string(tokens[0]) + " kind=syntax");
+        return false;
+    }
+
+    std::string decodedText;
+    URI::decode(text, decodedText);
+
+    std::unique_lock<std::mutex> lock(_docManager.getDocumentMutex());
+    getLOKitDocument()->setView(_viewId);
+    getLOKitDocument()->postWindowExtTextInputEvent(id, type, decodedText.c_str());
+
+    return true;
+}
+
+bool ChildSession::keyEvent(const char* /*buffer*/, int /*length*/,
+                            const std::vector<std::string>& tokens,
+                            const LokEventTargetEnum target)
 {
     int type, charcode, keycode;
-    if (tokens.size() != 4 ||
-        !getTokenKeyword(tokens[1], "type",
+    unsigned winId = 0;
+    unsigned counter = 1;
+    unsigned expectedTokens = 4; // cmdname(key), type, char, key are strictly required
+    if (target == LokEventTargetEnum::Window)
+    {
+        if (tokens.size() <= counter ||
+            !getTokenUInt32(tokens[counter++], "id", winId))
+        {
+            LOG_ERR("Window key event expects a valid id= attribute");
+            sendTextFrame("error: cmd=" + std::string(tokens[0]) + " kind=syntax");
+            return false;
+        }
+        else // id= attribute is found
+            expectedTokens++;
+    }
+
+    if (tokens.size() != expectedTokens ||
+        !getTokenKeyword(tokens[counter++], "type",
                          {{"input", LOK_KEYEVENT_KEYINPUT}, {"up", LOK_KEYEVENT_KEYUP}},
                          type) ||
-        !getTokenInteger(tokens[2], "char", charcode) ||
-        !getTokenInteger(tokens[3], "key", keycode))
+        !getTokenInteger(tokens[counter++], "char", charcode) ||
+        !getTokenInteger(tokens[counter++], "key", keycode))
     {
-        sendTextFrame("error: cmd=key kind=syntax");
+        sendTextFrame("error: cmd=" + std::string(tokens[0]) + "  kind=syntax");
         return false;
     }
 
     // Don't close LO window!
-    constexpr auto KEY_CTRL = 0x2000;
-    constexpr auto KEY_W = 0x0216;
+    constexpr int KEY_CTRL = 0x2000;
+    constexpr int KEY_W = 0x0216;
     if (keycode == (KEY_CTRL | KEY_W))
     {
         return true;
@@ -730,22 +837,25 @@ bool ChildSession::keyEvent(const char* /*buffer*/, int /*length*/, const std::v
 
     // Ctrl+Tab switching browser tabs,
     // Doesn't insert tabs.
-    constexpr auto KEY_TAB = 0x0502;
+    constexpr int KEY_TAB = 0x0502;
     if (keycode == (KEY_CTRL | KEY_TAB))
     {
         return true;
     }
 
     std::unique_lock<std::mutex> lock(_docManager.getDocumentMutex());
-
     getLOKitDocument()->setView(_viewId);
-
-    getLOKitDocument()->postKeyEvent(type, charcode, keycode);
+    if (target == LokEventTargetEnum::Document)
+        getLOKitDocument()->postKeyEvent(type, charcode, keycode);
+    else if (winId != 0)
+        getLOKitDocument()->postWindowKeyEvent(winId, type, charcode, keycode);
 
     return true;
 }
 
-bool ChildSession::mouseEvent(const char* /*buffer*/, int /*length*/, const std::vector<std::string>& tokens)
+bool ChildSession::mouseEvent(const char* /*buffer*/, int /*length*/,
+                              const std::vector<std::string>& tokens,
+                              const LokEventTargetEnum target)
 {
     int type, x, y, count;
     bool success = true;
@@ -754,38 +864,62 @@ bool ChildSession::mouseEvent(const char* /*buffer*/, int /*length*/, const std:
     int buttons = 1; // left button
     int modifier = 0;
 
-    if (tokens.size() < 5 ||
-        !getTokenKeyword(tokens[1], "type",
+    unsigned winId = 0;
+    unsigned counter = 1;
+    unsigned minTokens = 5; // cmdname(mouse), type, x, y, count are strictly required
+    if (target == LokEventTargetEnum::Window)
+    {
+        if (tokens.size() <= counter ||
+            !getTokenUInt32(tokens[counter++], "id", winId))
+        {
+            LOG_ERR("Window mouse event expects a valid id= attribute");
+            success = false;
+        }
+        else // id= attribute is found
+            minTokens++;
+    }
+
+    if (tokens.size() < minTokens ||
+        !getTokenKeyword(tokens[counter++], "type",
                          {{"buttondown", LOK_MOUSEEVENT_MOUSEBUTTONDOWN},
                           {"buttonup", LOK_MOUSEEVENT_MOUSEBUTTONUP},
                           {"move", LOK_MOUSEEVENT_MOUSEMOVE}},
                          type) ||
-        !getTokenInteger(tokens[2], "x", x) ||
-        !getTokenInteger(tokens[3], "y", y) ||
-        !getTokenInteger(tokens[4], "count", count))
+        !getTokenInteger(tokens[counter++], "x", x) ||
+        !getTokenInteger(tokens[counter++], "y", y) ||
+        !getTokenInteger(tokens[counter++], "count", count))
     {
         success = false;
     }
 
     // compatibility with older loleaflets
-    if (success && tokens.size() > 5 && !getTokenInteger(tokens[5], "buttons", buttons))
+    if (success && tokens.size() > counter && !getTokenInteger(tokens[counter++], "buttons", buttons))
         success = false;
 
     // compatibility with older loleaflets
-    if (success && tokens.size() > 6 && !getTokenInteger(tokens[6], "modifier", modifier))
+    if (success && tokens.size() > counter && !getTokenInteger(tokens[counter++], "modifier", modifier))
         success = false;
 
     if (!success)
     {
-        sendTextFrame("error: cmd=mouse kind=syntax");
+        sendTextFrame("error: cmd=" +  std::string(tokens[0]) + " kind=syntax");
         return false;
     }
 
     std::unique_lock<std::mutex> lock(_docManager.getDocumentMutex());
 
     getLOKitDocument()->setView(_viewId);
-
-    getLOKitDocument()->postMouseEvent(type, x, y, count, buttons, modifier);
+    switch (target)
+    {
+    case LokEventTargetEnum::Document:
+        getLOKitDocument()->postMouseEvent(type, x, y, count, buttons, modifier);
+        break;
+    case LokEventTargetEnum::Window:
+        getLOKitDocument()->postWindowMouseEvent(winId, type, x, y, count, buttons, modifier);
+        break;
+    default:
+        assert(false && "Unsupported mouse target type");
+    }
 
     return true;
 }
@@ -799,7 +933,10 @@ bool ChildSession::unoCommand(const char* /*buffer*/, int /*length*/, const std:
     }
 
     // we need to get LOK_CALLBACK_UNO_COMMAND_RESULT callback when saving
-    const bool bNotify = (tokens[1] == ".uno:Save" || tokens[1] == ".uno:Undo" || tokens[1] == ".uno:Redo");
+    const bool bNotify = (tokens[1] == ".uno:Save" ||
+                          tokens[1] == ".uno:Undo" ||
+                          tokens[1] == ".uno:Redo" ||
+                          Util::startsWith(tokens[1], "vnd.sun.star.script:"));
 
     std::unique_lock<std::mutex> lock(_docManager.getDocumentMutex());
 
@@ -848,6 +985,92 @@ bool ChildSession::selectText(const char* /*buffer*/, int /*length*/, const std:
     return true;
 }
 
+bool ChildSession::renderWindow(const char* /*buffer*/, int /*length*/, const std::vector<std::string>& tokens)
+{
+    std::unique_lock<std::mutex> lock(_docManager.getDocumentMutex());
+    getLOKitDocument()->setView(_viewId);
+
+    unsigned winId = 0;
+    if (tokens.size() > 1)
+    {
+        std::istringstream reader(tokens[1]);
+        reader >> winId;
+    }
+
+    int startX = 0, startY = 0;
+    int bufferWidth = 800, bufferHeight = 600;
+    std::string paintRectangle;
+    if (tokens.size() > 2 && getTokenString(tokens[2], "rectangle", paintRectangle))
+    {
+        const std::vector<std::string> rectParts = LOOLProtocol::tokenize(paintRectangle.c_str(), paintRectangle.length(), ',');
+        startX = std::atoi(rectParts[0].c_str());
+        startY = std::atoi(rectParts[1].c_str());
+        bufferWidth = std::atoi(rectParts[2].c_str());
+        bufferHeight = std::atoi(rectParts[3].c_str());
+    }
+    else
+        LOG_WRN("windowpaint command doesn't specify a rectangle= attribute.");
+
+    size_t pixmapDataSize = 4 * bufferWidth * bufferHeight;
+    std::vector<unsigned char> pixmap(pixmapDataSize);
+    int width = bufferWidth, height = bufferHeight;
+    std::string response;
+    Timestamp timestamp;
+    getLOKitDocument()->paintWindow(winId, pixmap.data(), startX, startY, width, height);
+    const double area = width * height;
+    LOG_TRC("paintWindow for " << winId << " returned " << width << "X" << height
+            << "@(" << startX << "," << startY << ")"
+            << "and rendered in " << (timestamp.elapsed()/1000.)
+            << "ms (" << area / (timestamp.elapsed()) << " MP/s).");
+
+    response = "windowpaint: id=" + tokens[1] +
+        " width=" + std::to_string(width) + " height=" + std::to_string(height);
+
+    if (!paintRectangle.empty())
+        response += " rectangle=" + paintRectangle;
+
+    response += "\n";
+
+    std::vector<char> output;
+    output.reserve(response.size() + pixmapDataSize);
+    output.resize(response.size());
+    std::memcpy(output.data(), response.data(), response.size());
+
+    const auto mode = static_cast<LibreOfficeKitTileMode>(getLOKitDocument()->getTileMode());
+
+    // TODO: use png cache for dialogs too
+    if (!Png::encodeSubBufferToPNG(pixmap.data(), 0, 0, width, height, bufferWidth, bufferHeight, output, mode))
+    {
+        LOG_ERR("Failed to encode into PNG.");
+        return false;
+    }
+
+    LOG_TRC("Sending response (" << output.size() << " bytes) for: " << response);
+    sendBinaryFrame(output.data(), output.size());
+    return true;
+}
+
+
+bool ChildSession::sendWindowCommand(const char* /*buffer*/, int /*length*/, const std::vector<std::string>& tokens)
+{
+    std::unique_lock<std::mutex> lock(_docManager.getDocumentMutex());
+    getLOKitDocument()->setView(_viewId);
+
+    unsigned winId = 0;
+    if (tokens.size() > 1)
+    {
+        std::istringstream reader(tokens[1]);
+        reader >> winId;
+    }
+
+    getLOKitDocument()->setView(_viewId);
+
+    if (tokens.size() > 2 && tokens[2] == "close")
+        getLOKitDocument()->postWindow(winId, LOK_WINDOW_CLOSE);
+
+    return true;
+}
+
 bool ChildSession::selectGraphic(const char* /*buffer*/, int /*length*/, const std::vector<std::string>& tokens)
 {
     int type, x, y;
@@ -935,6 +1158,12 @@ bool ChildSession::saveAs(const char* /*buffer*/, int /*length*/, const std::vec
 
         getLOKitDocument()->setView(_viewId);
 
+        if (filterOptions.empty() && format == "html")
+        {
+            // Opt-in to avoid linked images, those would not leave the chroot.
+            filterOptions = "EmbedImages";
+        }
+
         LOG_DBG("Calling LOK's saveAs with: '" << url.c_str() << "', '" <<
                 (format.size() == 0 ? "(nullptr)" : format.c_str()) << "', '" <<
                 (filterOptions.size() == 0 ? "(nullptr)" : filterOptions.c_str()) << "'.");
@@ -1024,7 +1253,7 @@ void ChildSession::rememberEventsForInactiveUser(const int type, const std::stri
 {
     if (type == LOK_CALLBACK_INVALIDATE_TILES)
     {
-        auto lock(getLock());
+        std::unique_lock<std::mutex> lock(getLock());
         _stateRecorder.recordInvalidate(); // TODO remember the area, not just a bool ('true' invalidates everything)
     }
     else if (type == LOK_CALLBACK_INVALIDATE_VISIBLE_CURSOR ||
@@ -1039,7 +1268,7 @@ void ChildSession::rememberEventsForInactiveUser(const int type, const std::stri
              type == LOK_CALLBACK_INVALIDATE_HEADER ||
              type == LOK_CALLBACK_CELL_ADDRESS)
     {
-        auto lock(getLock());
+        std::unique_lock<std::mutex> lock(getLock());
         _stateRecorder.recordEvent(type, payload);
     }
     else if (type == LOK_CALLBACK_INVALIDATE_VIEW_CURSOR ||
@@ -1049,10 +1278,10 @@ void ChildSession::rememberEventsForInactiveUser(const int type, const std::stri
              type == LOK_CALLBACK_VIEW_CURSOR_VISIBLE ||
              type == LOK_CALLBACK_VIEW_LOCK)
     {
-        auto lock(getLock());
+        std::unique_lock<std::mutex> lock(getLock());
         Poco::JSON::Parser parser;
 
-        auto root = parser.parse(payload).extract<Poco::JSON::Object::Ptr>();
+        Poco::JSON::Object::Ptr root = parser.parse(payload).extract<Poco::JSON::Object::Ptr>();
         int viewId = root->getValue<int>("viewId");
         _stateRecorder.recordViewEvent(viewId, type, payload);
     }
@@ -1062,7 +1291,7 @@ void ChildSession::rememberEventsForInactiveUser(const int type, const std::stri
         std::string value;
         if (LOOLProtocol::parseNameValuePair(payload, name, value, '='))
         {
-            auto lock(getLock());
+            std::unique_lock<std::mutex> lock(getLock());
             _stateRecorder.recordState(name, payload);
         }
     }
@@ -1070,40 +1299,42 @@ void ChildSession::rememberEventsForInactiveUser(const int type, const std::stri
              type == LOK_CALLBACK_REDLINE_TABLE_ENTRY_MODIFIED ||
              type == LOK_CALLBACK_COMMENT)
     {
-        auto lock(getLock());
+        std::unique_lock<std::mutex> lock(getLock());
         _stateRecorder.recordEventSequence(type, payload);
     }
 }
 
-void ChildSession::updateSpeed() {
-
+void ChildSession::updateSpeed()
+{
     std::chrono::steady_clock::time_point now(std::chrono::steady_clock::now());
 
-    while(_cursorInvalidatedEvent.size() != 0 &&
-        std::chrono::duration_cast<std::chrono::milliseconds>(now - _cursorInvalidatedEvent.front()).count() > _eventStorageIntervalMs)
+    while (_cursorInvalidatedEvent.size() != 0 &&
+           std::chrono::duration_cast<std::chrono::milliseconds>(now - _cursorInvalidatedEvent.front()).count() > _eventStorageIntervalMs)
     {
         _cursorInvalidatedEvent.pop();
     }
+
     _cursorInvalidatedEvent.push(now);
     _docManager.updateEditorSpeeds(_viewId, _cursorInvalidatedEvent.size());
 }
 
-int ChildSession::getSpeed() {
-
+int ChildSession::getSpeed()
+{
     std::chrono::steady_clock::time_point now(std::chrono::steady_clock::now());
 
-    while(_cursorInvalidatedEvent.size() > 0 &&
-        std::chrono::duration_cast<std::chrono::milliseconds>(now - _cursorInvalidatedEvent.front()).count() > _eventStorageIntervalMs)
+    while (_cursorInvalidatedEvent.size() > 0 &&
+           std::chrono::duration_cast<std::chrono::milliseconds>(now - _cursorInvalidatedEvent.front()).count() > _eventStorageIntervalMs)
     {
         _cursorInvalidatedEvent.pop();
     }
+
     return _cursorInvalidatedEvent.size();
 }
 
 void ChildSession::loKitCallback(const int type, const std::string& payload)
 {
-    const auto typeName = LOKitHelper::kitCallbackTypeToString(type);
-    LOG_TRC("CallbackWorker::callback [" << getName() << "]: " <<
+    const std::string typeName = LOKitHelper::kitCallbackTypeToString(type);
+    LOG_TRC("ChildSession::loKitCallback [" << getName() << "]: " <<
             typeName << " [" << payload << "].");
 
     if (isCloseFrame())
@@ -1299,6 +1530,12 @@ void ChildSession::loKitCallback(const int type, const std::string& payload)
     case LOK_CALLBACK_RULER_UPDATE:
         sendTextFrame("rulerupdate: " + payload);
         break;
+    case LOK_CALLBACK_WINDOW:
+        sendTextFrame("window: " + payload);
+        break;
+    case LOK_CALLBACK_VALIDITY_LIST_BUTTON:
+        sendTextFrame("validitylistbutton: " + payload);
+        break;
     default:
         LOG_ERR("Unknown callback event (" << type << "): " << payload);
     }
