@@ -26,7 +26,7 @@ L.Control.PermissionSwitch = L.Control.extend({
 		else {
 			this._map.setPermission('view');
 		}
-		this._refocusOnMap();
+		this._map.focus();
 	},
 
 	_onUpdatePermission: function (e) {
