@@ -23,7 +23,7 @@ L.GridLayer = L.Layer.extend({
 	},
 
 	initialize: function (options) {
-		options = L.setOptions(this, options);
+		L.setOptions(this, options);
 	},
 
 	onAdd: function () {
@@ -368,22 +368,25 @@ L.GridLayer = L.Layer.extend({
 			return;
 		}
 		if (!zoom) {
-			var zoom = this._map.getZoom();
+			zoom = this._map.getZoom();
 		}
 		var docPixelLimits = new L.Point(this._docWidthTwips / this.options.tileWidthTwips,
 			this._docHeightTwips / this.options.tileHeightTwips);
-		docPixelLimits = extraSize ? docPixelLimits.multiplyBy(this._tileSize).add(extraSize) :
-			docPixelLimits.multiplyBy(this._tileSize);
-
+		docPixelLimits = docPixelLimits.multiplyBy(this._tileSize);
 		var scale = this._map.getZoomScale(zoom, 10);
 		var topLeft = new L.Point(0, 0);
 		topLeft = this._map.unproject(topLeft.multiplyBy(scale));
 		var bottomRight = new L.Point(docPixelLimits.x, docPixelLimits.y);
-		bottomRight = this._map.unproject(bottomRight.multiplyBy(scale));
+		bottomRight = bottomRight.multiplyBy(scale);
+		if (extraSize) {
+			bottomRight = bottomRight.add(extraSize);
+		}
+		bottomRight = this._map.unproject(bottomRight);
 
 		if (this._documentInfo === '' || sizeChanged) {
 			// we just got the first status so we need to center the document
 			this._map.setMaxBounds(new L.LatLngBounds(topLeft, bottomRight), options);
+			this._map.setDocBounds(new L.LatLngBounds(topLeft, this._map.unproject(docPixelLimits.multiplyBy(scale))));
 		}
 
 		var scrollPixelLimits = new L.Point(this._docWidthTwips / this._tileWidthTwips,
@@ -391,7 +394,7 @@ L.GridLayer = L.Layer.extend({
 		scrollPixelLimits = extraSize ? scrollPixelLimits.multiplyBy(this._tileSize).add(extraSize.multiplyBy(scale)) :
 			scrollPixelLimits.multiplyBy(this._tileSize);
 		this._docPixelSize = {x: scrollPixelLimits.x, y: scrollPixelLimits.y};
-		this._map.fire('docsize', {x: scrollPixelLimits.x, y: scrollPixelLimits.y});
+		this._map.fire('docsize', {x: scrollPixelLimits.x, y: scrollPixelLimits.y, extraSize: extraSize});
 	},
 
 	_checkSpreadSheetBounds: function (newZoom) {
@@ -539,18 +542,7 @@ L.GridLayer = L.Layer.extend({
 			if (newView) {
 				// we know that a new set of tiles that cover the whole view has been requested
 				// so we're able to cancel the previous requests that are being processed
-				this._map._socket.sendMessage('canceltiles');
-				for (key in this._tiles) {
-					if (!this._tiles[key].loaded) {
-						L.DomUtil.remove(this._tiles[key].el);
-						delete this._tiles[key];
-						if (this._debug) {
-							this._debugCancelledTiles++;
-							this._debugShowTileData();
-						}
-					}
-				}
-				this._emptyTilesCount = 0;
+				this._cancelTiles();
 			}
 
 			// if its the first batch of tiles to load
@@ -611,19 +603,7 @@ L.GridLayer = L.Layer.extend({
 			if (newView) {
 				// we know that a new set of tiles that cover the whole view has been requested
 				// so we're able to cancel the previous requests that are being processed
-				this._map._socket.sendMessage('canceltiles');
-				for (key in this._tiles) {
-					tile = this._tiles[key];
-					if (!tile.loaded) {
-						L.DomUtil.remove(tile.el);
-						delete this._tiles[key];
-						if (this._debug && this._debugDataCancelledTiles) {
-							this._debugCancelledTiles++;
-							this._debugDataCancelledTiles.setPrefix('Cancelled tiles: ' + this._debugCancelledTiles);
-						}
-					}
-				}
-				this._emptyTilesCount = 0;
+				this._cancelTiles();
 			}
 
 			// if its the first batch of tiles to load
@@ -704,6 +684,46 @@ L.GridLayer = L.Layer.extend({
 		}
 	},
 
+	_cancelTiles: function() {
+		this._map._socket.sendMessage('canceltiles');
+		for (var key in this._tiles) {
+			var tile = this._tiles[key];
+			// When _invalidCount > 0 the tile has been invalidated, however the new tile content
+			// has not yet been fetched and because of `canceltiles` message it will never be
+			// so we need to remove the tile, or when the tile is back inside the visible area
+			// its content would be the old invalidated one. Drop only those tiles which are not in
+			// the new visible area.
+			// example: a tile is invalidated but a sudden scroll to the cell cursor position causes
+			// to move the tile out of the visible area before the new content is fetched
+
+			var dropTile = !tile.loaded;
+			var coords = tile.coords;
+			if (coords.part === this._selectedPart) {
+				var visibleTopLeft = this._latLngToTwips(this._map.getBounds().getNorthWest());
+				var visibleBottomRight = this._latLngToTwips(this._map.getBounds().getSouthEast());
+				var visibleArea = new L.Bounds(visibleTopLeft, visibleBottomRight);
+				var tileTopLeft = this._coordsToTwips(coords);
+				var tileBottomRight = new L.Point(this._tileWidthTwips, this._tileHeightTwips);
+				var tileBounds = new L.Bounds(tileTopLeft, tileTopLeft.add(tileBottomRight));
+				dropTile |= (tile._invalidCount > 0 && !visibleArea.intersects(tileBounds));
+			}
+			else {
+				dropTile |= tile._invalidCount > 0;
+			}
+
+
+			if (dropTile) {
+				L.DomUtil.remove(tile.el);
+				delete this._tiles[key];
+				if (this._debug && this._debugDataCancelledTiles) {
+					this._debugCancelledTiles++;
+					this._debugDataCancelledTiles.setPrefix('Cancelled tiles: ' + this._debugCancelledTiles);
+				}
+			}
+		}
+		this._emptyTilesCount = 0;
+	},
+
 	_isValidTile: function (coords) {
 		if (coords.x < 0 || coords.y < 0) {
 			return false;
