@@ -7,7 +7,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include "Storage.hpp"
 
@@ -40,13 +40,14 @@
 #include <Poco/Net/SSLManager.h>
 
 #include "Auth.hpp"
-#include "Common.hpp"
+#include <Common.hpp>
 #include "Exceptions.hpp"
 #include "LOOLWSD.hpp"
-#include "Log.hpp"
-#include "Unit.hpp"
-#include "Util.hpp"
-#include "common/FileUtil.hpp"
+#include <Log.hpp>
+#include <Unit.hpp>
+#include <Util.hpp>
+#include <common/FileUtil.hpp>
+#include <common/JsonUtil.hpp>
 
 bool StorageBase::FilesystemEnabled;
 bool StorageBase::WopiEnabled;
@@ -54,7 +55,7 @@ Util::RegexListMatcher StorageBase::WopiHosts;
 
 std::string StorageBase::getLocalRootPath() const
 {
-    auto localPath = _jailPath;
+    std::string localPath = _jailPath;
     if (localPath[0] == '/')
     {
         // Remove the leading /
@@ -62,7 +63,7 @@ std::string StorageBase::getLocalRootPath() const
     }
 
     // /chroot/jailId/user/doc/childId
-    const auto rootPath = Poco::Path(_localStorePath, localPath);
+    const Poco::Path rootPath = Poco::Path(_localStorePath, localPath);
     Poco::File(rootPath).createDirectories();
 
     return rootPath.toString();
@@ -86,7 +87,7 @@ void StorageBase::initialize()
         for (size_t i = 0; ; ++i)
         {
             const std::string path = "storage.wopi.host[" + std::to_string(i) + "]";
-            const auto host = app.config().getString(path, "");
+            const std::string host = app.config().getString(path, "");
             if (!host.empty())
             {
                 if (app.config().getBool(path + "[@allow]", false))
@@ -115,7 +116,7 @@ void StorageBase::initialize()
     // Init client
     Poco::Net::Context::Params sslClientParams;
 
-    // TODO: Be more strict and setup SSL key/certs for remove server and us
+    // TODO: Be more strict and setup SSL key/certs for remote server and us
     sslClientParams.verificationMode = Poco::Net::Context::VERIFY_NONE;
 
     Poco::SharedPtr<Poco::Net::PrivateKeyPassphraseHandler> consoleClientHandler = new Poco::Net::KeyConsoleHandler(false);
@@ -222,7 +223,7 @@ std::unique_ptr<StorageBase> StorageBase::create(const Poco::URI& uri, const std
         {
             return std::unique_ptr<StorageBase>(new WopiStorage(uri, jailRoot, jailPath));
         }
-
+        LOG_ERR("No acceptable WOPI hosts found matching the target host [" << targetHost << "] in config.");
         throw UnauthorizedRequestException("No acceptable WOPI hosts found matching the target host [" + targetHost + "] in config.");
     }
 
@@ -233,13 +234,13 @@ std::atomic<unsigned> LocalStorage::LastLocalStorageId;
 
 std::unique_ptr<LocalStorage::LocalFileInfo> LocalStorage::getLocalFileInfo()
 {
-    const auto path = Poco::Path(_uri.getPath());
+    const Poco::Path path = Poco::Path(_uri.getPath());
     LOG_DBG("Getting info for local uri [" << _uri.toString() << "], path [" << path.toString() << "].");
 
     const auto& filename = path.getFileName();
-    const auto file = Poco::File(path);
-    const auto lastModified = file.getLastModified();
-    const auto size = file.getSize();
+    const Poco::File file = Poco::File(path);
+    const Poco::Timestamp lastModified = file.getLastModified();
+    const size_t size = file.getSize();
 
     _fileInfo = FileInfo({filename, "localhost", lastModified, size});
 
@@ -250,13 +251,13 @@ std::unique_ptr<LocalStorage::LocalFileInfo> LocalStorage::getLocalFileInfo()
 std::string LocalStorage::loadStorageFileToLocal(const Authorization& /*auth*/)
 {
     // /chroot/jailId/user/doc/childId/file.ext
-    const auto filename = Poco::Path(_uri.getPath()).getFileName();
+    const std::string filename = Poco::Path(_uri.getPath()).getFileName();
     _jailedFilePath = Poco::Path(getLocalRootPath(), filename).toString();
     LOG_INF("Public URI [" << _uri.getPath() <<
             "] jailed to [" << _jailedFilePath << "].");
 
     // Despite the talk about URIs it seems that _uri is actually just a pathname here
-    const auto publicFilePath = _uri.getPath();
+    const std::string publicFilePath = _uri.getPath();
 
     if (!FileUtil::checkDiskSpace(_jailedFilePath))
     {
@@ -267,7 +268,8 @@ std::string LocalStorage::loadStorageFileToLocal(const Authorization& /*auth*/)
     if (!Poco::File(_jailedFilePath).exists() && link(publicFilePath.c_str(), _jailedFilePath.c_str()) == -1)
     {
         // Failed
-        LOG_WRN("link(\"" << publicFilePath << "\", \"" << _jailedFilePath << "\") failed. Will copy.");
+        LOG_WRN("link(\"" << publicFilePath << "\", \"" << _jailedFilePath << "\") failed. Will copy. "
+                "Linking error: " << errno << " " << strerror(errno));
     }
 
     try
@@ -302,7 +304,7 @@ StorageBase::SaveResult LocalStorage::saveLocalFileToStorage(const Authorization
 {
     try
     {
-        LOG_TRC("Saving local file to local file storage " << _isCopy << " for " << _jailedFilePath);
+        LOG_TRC("Saving local file to local file storage (isCopy: " << _isCopy << ") for " << _jailedFilePath);
         // Copy the file back.
         if (_isCopy && Poco::File(_jailedFilePath).exists())
         {
@@ -313,7 +315,7 @@ StorageBase::SaveResult LocalStorage::saveLocalFileToStorage(const Authorization
         // update its fileinfo object. This is used later to check if someone else changed the
         // document while we are/were editing it
         _fileInfo._modifiedTime = Poco::File(_uri.getPath()).getLastModified();
-        Log::trace() << "New FileInfo modified time in storage " << _fileInfo._modifiedTime << Log::end;
+        LOG_TRC("New FileInfo modified time in storage " << _fileInfo._modifiedTime);
     }
     catch (const Poco::Exception& exc)
     {
@@ -325,7 +327,8 @@ StorageBase::SaveResult LocalStorage::saveLocalFileToStorage(const Authorization
     return StorageBase::SaveResult(StorageBase::SaveResult::OK);
 }
 
-namespace {
+namespace
+{
 
 inline
 Poco::Net::HTTPClientSession* getHTTPClientSession(const Poco::URI& uri)
@@ -338,107 +341,6 @@ Poco::Net::HTTPClientSession* getHTTPClientSession(const Poco::URI& uri)
         : new Poco::Net::HTTPClientSession(uri.getHost(), uri.getPort());
 }
 
-int getLevenshteinDist(const std::string& string1, const std::string& string2) {
-    int matrix[string1.size() + 1][string2.size() + 1];
-    std::memset(matrix, 0, sizeof(matrix[0][0]) * (string1.size() + 1) * (string2.size() + 1));
-
-    for (size_t i = 0; i < string1.size() + 1; i++)
-    {
-        for (size_t j = 0; j < string2.size() + 1; j++)
-        {
-            if (i == 0)
-            {
-                matrix[i][j] = j;
-            }
-            else if (j == 0)
-            {
-                matrix[i][j] = i;
-            }
-            else if (string1[i - 1] == string2[j - 1])
-            {
-                matrix[i][j] = matrix[i - 1][j - 1];
-            }
-            else
-            {
-                matrix[i][j] = 1 + std::min(std::min(matrix[i][j - 1], matrix[i - 1][j]),
-                                            matrix[i - 1][j - 1]);
-            }
-        }
-    }
-
-    return matrix[string1.size()][string2.size()];
-}
-
-// Gets value for `key` directly from the given JSON in `object`
-template <typename T>
-T getJSONValue(const Poco::JSON::Object::Ptr &object, const std::string& key)
-{
-    T value = T();
-    try
-    {
-        const Poco::Dynamic::Var valueVar = object->get(key);
-        value = valueVar.convert<T>();
-    }
-    catch (const Poco::Exception& exc)
-    {
-        LOG_ERR("getJSONValue: " << exc.displayText() <<
-                (exc.nested() ? " (" + exc.nested()->displayText() + ")" : ""));
-    }
-
-    return value;
-}
-
-// Function that searches `object` for `key` and warns if there are minor mis-spellings involved
-// Upon successfull search, fills `value` with value found in object.
-template <typename T>
-void getWOPIValue(const Poco::JSON::Object::Ptr &object, const std::string& key, T& value)
-{
-    std::vector<std::string> propertyNames;
-    object->getNames(propertyNames);
-
-    // Check each property name against given key
-    // and accept with a mis-spell tolerance of 2
-    // TODO: propertyNames can be pruned after getting its value
-    for (const auto& userInput: propertyNames)
-    {
-        std::string string1(key), string2(userInput);
-        std::transform(key.begin(), key.end(), string1.begin(), tolower);
-        std::transform(userInput.begin(), userInput.end(), string2.begin(), tolower);
-        int levDist = getLevenshteinDist(string1, string2);
-
-        if (levDist > 2) /* Mis-spelling tolerance */
-            continue;
-        else if (levDist > 0 || key != userInput)
-        {
-            LOG_WRN("Incorrect JSON property [" << userInput << "]. Did you mean " << key << " ?");
-            return;
-        }
-
-        value = getJSONValue<T>(object, userInput);
-        return;
-    }
-
-    LOG_WRN("Missing JSON property [" << key << "]");
-}
-
-// Parse the json string and fill the Poco::JSON object
-// Returns true if parsing successful otherwise false
-bool parseJSON(const std::string& json, Poco::JSON::Object::Ptr& object)
-{
-    bool success = false;
-    const auto index = json.find_first_of('{');
-    if (index != std::string::npos)
-    {
-        const std::string stringJSON = json.substr(index);
-        Poco::JSON::Parser parser;
-        const auto result = parser.parse(stringJSON);
-        object = result.extract<Poco::JSON::Object::Ptr>();
-        success = true;
-    }
-
-    return success;
-}
-
 void addStorageDebugCookie(Poco::Net::HTTPRequest& request)
 {
     (void) request;
@@ -457,7 +359,7 @@ void addStorageDebugCookie(Poco::Net::HTTPRequest& request)
 #endif
 }
 
-Poco::Timestamp iso8601ToTimestamp(const std::string& iso8601Time)
+Poco::Timestamp iso8601ToTimestamp(const std::string& iso8601Time, const std::string& name)
 {
     Poco::Timestamp timestamp = Poco::Timestamp::fromEpochTime(0);
     try
@@ -469,8 +371,8 @@ Poco::Timestamp iso8601ToTimestamp(const std::string& iso8601Time)
     }
     catch (const Poco::SyntaxException& exc)
     {
-        LOG_WRN("Time [" << iso8601Time << "] is in invalid format: " << exc.displayText() <<
-                (exc.nested() ? " (" + exc.nested()->displayText() + ")" : ""));
+        LOG_WRN(name << " [" << iso8601Time << "] is in invalid format: " << exc.displayText() <<
+                (exc.nested() ? " (" + exc.nested()->displayText() + ")" : "") << ". Returning " << timestamp);
     }
 
     return timestamp;
@@ -503,7 +405,7 @@ std::unique_ptr<WopiStorage::WOPIFileInfo> WopiStorage::getWOPIFileInfo(const Au
         std::istream& rs = psession->receiveResponse(response);
         callDuration = (std::chrono::steady_clock::now() - startTime);
 
-        auto logger = Log::trace();
+        Log::StreamLogger logger = Log::trace();
         if (logger.enabled())
         {
             logger << "WOPI::CheckFileInfo header for URI [" << uriObject.toString() << "]:\n";
@@ -544,6 +446,7 @@ std::unique_ptr<WopiStorage::WOPIFileInfo> WopiStorage::getWOPIFileInfo(const Au
     bool hidePrintOption = false;
     bool hideSaveOption = false;
     bool hideExportOption = false;
+    bool hideChangeTrackingControls = false;
     bool disablePrint = false;
     bool disableExport = false;
     bool disableCopy = false;
@@ -553,27 +456,28 @@ std::unique_ptr<WopiStorage::WOPIFileInfo> WopiStorage::getWOPIFileInfo(const Au
 
     LOG_DBG("WOPI::CheckFileInfo returned: " << resMsg << ". Call duration: " << callDuration.count() << "s");
     Poco::JSON::Object::Ptr object;
-    if (parseJSON(resMsg, object))
+    if (JsonUtil::parseJSON(resMsg, object))
     {
-        getWOPIValue(object, "BaseFileName", filename);
-        getWOPIValue(object, "Size", size);
-        getWOPIValue(object, "OwnerId", ownerId);
-        getWOPIValue(object, "UserId", userId);
-        getWOPIValue(object, "UserFriendlyName", userName);
-        getWOPIValue(object, "UserExtraInfo", userExtraInfo);
-        getWOPIValue(object, "WatermarkText", watermarkText);
-        getWOPIValue(object, "UserCanWrite", canWrite);
-        getWOPIValue(object, "PostMessageOrigin", postMessageOrigin);
-        getWOPIValue(object, "HidePrintOption", hidePrintOption);
-        getWOPIValue(object, "HideSaveOption", hideSaveOption);
-        getWOPIValue(object, "HideExportOption", hideExportOption);
-        getWOPIValue(object, "EnableOwnerTermination", enableOwnerTermination);
-        getWOPIValue(object, "DisablePrint", disablePrint);
-        getWOPIValue(object, "DisableExport", disableExport);
-        getWOPIValue(object, "DisableCopy", disableCopy);
-        getWOPIValue(object, "DisableInactiveMessages", disableInactiveMessages);
-        getWOPIValue(object, "LastModifiedTime", lastModifiedTime);
-        getWOPIValue(object, "UserCanNotWriteRelative", userCanNotWriteRelative);
+        JsonUtil::findJSONValue(object, "BaseFileName", filename);
+        JsonUtil::findJSONValue(object, "Size", size);
+        JsonUtil::findJSONValue(object, "OwnerId", ownerId);
+        JsonUtil::findJSONValue(object, "UserId", userId);
+        JsonUtil::findJSONValue(object, "UserFriendlyName", userName);
+        JsonUtil::findJSONValue(object, "UserExtraInfo", userExtraInfo);
+        JsonUtil::findJSONValue(object, "WatermarkText", watermarkText);
+        JsonUtil::findJSONValue(object, "UserCanWrite", canWrite);
+        JsonUtil::findJSONValue(object, "PostMessageOrigin", postMessageOrigin);
+        JsonUtil::findJSONValue(object, "HidePrintOption", hidePrintOption);
+        JsonUtil::findJSONValue(object, "HideSaveOption", hideSaveOption);
+        JsonUtil::findJSONValue(object, "HideExportOption", hideExportOption);
+        JsonUtil::findJSONValue(object, "hideChangeTrackingControls", hideChangeTrackingControls);
+        JsonUtil::findJSONValue(object, "EnableOwnerTermination", enableOwnerTermination);
+        JsonUtil::findJSONValue(object, "DisablePrint", disablePrint);
+        JsonUtil::findJSONValue(object, "DisableExport", disableExport);
+        JsonUtil::findJSONValue(object, "DisableCopy", disableCopy);
+        JsonUtil::findJSONValue(object, "DisableInactiveMessages", disableInactiveMessages);
+        JsonUtil::findJSONValue(object, "LastModifiedTime", lastModifiedTime);
+        JsonUtil::findJSONValue(object, "UserCanNotWriteRelative", userCanNotWriteRelative);
     }
     else
     {
@@ -581,10 +485,10 @@ std::unique_ptr<WopiStorage::WOPIFileInfo> WopiStorage::getWOPIFileInfo(const Au
         throw UnauthorizedRequestException("Access denied. WOPI::CheckFileInfo failed on: " + uriObject.toString());
     }
 
-    const Poco::Timestamp modifiedTime = iso8601ToTimestamp(lastModifiedTime);
+    const Poco::Timestamp modifiedTime = iso8601ToTimestamp(lastModifiedTime, "LastModifiedTime");
     _fileInfo = FileInfo({filename, ownerId, modifiedTime, size});
 
-    return std::unique_ptr<WopiStorage::WOPIFileInfo>(new WOPIFileInfo({userId, userName, userExtraInfo, watermarkText, canWrite, postMessageOrigin, hidePrintOption, hideSaveOption, hideExportOption, enableOwnerTermination, disablePrint, disableExport, disableCopy, disableInactiveMessages, userCanNotWriteRelative, callDuration}));
+    return std::unique_ptr<WopiStorage::WOPIFileInfo>(new WOPIFileInfo({userId, userName, userExtraInfo, watermarkText, canWrite, postMessageOrigin, hidePrintOption, hideSaveOption, hideExportOption, hideChangeTrackingControls, enableOwnerTermination, disablePrint, disableExport, disableCopy, disableInactiveMessages, userCanNotWriteRelative, callDuration}));
 }
 
 /// uri format: http://server/<...>/wopi*/files/<id>/content
@@ -614,7 +518,7 @@ std::string WopiStorage::loadStorageFileToLocal(const Authorization& auth)
         const std::chrono::duration<double> diff = (std::chrono::steady_clock::now() - startTime);
         _wopiLoadDuration += diff;
 
-        auto logger = Log::trace();
+        Log::StreamLogger logger = Log::trace();
         if (logger.enabled())
         {
             logger << "WOPI::GetFile header for URI [" << uriObject.toString() << "]:\n";
@@ -664,7 +568,7 @@ StorageBase::SaveResult WopiStorage::saveLocalFileToStorage(const Authorization&
     const bool isSaveAs = !saveAsPath.empty() && !saveAsFilename.empty();
     const std::string filePath(isSaveAs? saveAsPath: _jailedFilePath);
 
-    const auto size = getFileSize(filePath);
+    const size_t size = getFileSize(filePath);
 
     Poco::URI uriObject(_uri);
     uriObject.setPath(isSaveAs? uriObject.getPath(): uriObject.getPath() + "/contents");
@@ -746,7 +650,7 @@ StorageBase::SaveResult WopiStorage::saveLocalFileToStorage(const Authorization&
         std::istream& rs = psession->receiveResponse(response);
         Poco::StreamCopier::copyStream(rs, oss);
 
-        std::string wopiLog(isSaveAs? "WOPI::PutRelativeFile": "WOPI::PutFile");
+        const std::string wopiLog(isSaveAs ? "WOPI::PutRelativeFile" : "WOPI::PutFile");
         LOG_INF(wopiLog << " response: " << oss.str());
         LOG_INF(wopiLog << " uploaded " << size << " bytes from [" << filePath <<
                 "] -> [" << uriObject.toString() << "]: " <<
@@ -756,18 +660,18 @@ StorageBase::SaveResult WopiStorage::saveLocalFileToStorage(const Authorization&
         {
             saveResult.setResult(StorageBase::SaveResult::OK);
             Poco::JSON::Object::Ptr object;
-            if (parseJSON(oss.str(), object))
+            if (JsonUtil::parseJSON(oss.str(), object))
             {
-                const std::string lastModifiedTime = getJSONValue<std::string>(object, "LastModifiedTime");
+                const std::string lastModifiedTime = JsonUtil::getJSONValue<std::string>(object, "LastModifiedTime");
                 LOG_TRC(wopiLog << " returns LastModifiedTime [" << lastModifiedTime << "].");
-                _fileInfo._modifiedTime = iso8601ToTimestamp(lastModifiedTime);
+                _fileInfo._modifiedTime = iso8601ToTimestamp(lastModifiedTime, "LastModifiedTime");
 
                 if (isSaveAs)
                 {
-                    const std::string name = getJSONValue<std::string>(object, "Name");
+                    const std::string name = JsonUtil::getJSONValue<std::string>(object, "Name");
                     LOG_TRC(wopiLog << " returns Name [" << name << "].");
 
-                    const std::string url = getJSONValue<std::string>(object, "Url");
+                    const std::string url = JsonUtil::getJSONValue<std::string>(object, "Url");
                     LOG_TRC(wopiLog << " returns Url [" << url << "].");
 
                     saveResult.setSaveAsResult(name, url);
@@ -794,9 +698,9 @@ StorageBase::SaveResult WopiStorage::saveLocalFileToStorage(const Authorization&
         {
             saveResult.setResult(StorageBase::SaveResult::CONFLICT);
             Poco::JSON::Object::Ptr object;
-            if (parseJSON(oss.str(), object))
+            if (JsonUtil::parseJSON(oss.str(), object))
             {
-                const unsigned loolStatusCode = getJSONValue<unsigned>(object, "LOOLStatusCode");
+                const unsigned loolStatusCode = JsonUtil::getJSONValue<unsigned>(object, "LOOLStatusCode");
                 if (loolStatusCode == static_cast<unsigned>(LOOLStatusCode::DOC_CHANGED))
                 {
                     saveResult.setResult(StorageBase::SaveResult::DOC_CHANGED);
