@@ -11,10 +11,10 @@
  * Stub missing symbols required for unit tests ...
  */
 
-#include "config.h"
+#include <config.h>
 
 #include "DocumentBroker.hpp"
 
-void DocumentBroker::assertCorrectThread() {}
+void DocumentBroker::assertCorrectThread() const {}
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
