@@ -7,7 +7,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include "TileCache.hpp"
 
@@ -17,7 +17,6 @@
 #include <fstream>
 #include <iostream>
 #include <memory>
-#include <mutex>
 #include <sstream>
 #include <string>
 #include <vector>
@@ -32,20 +31,20 @@
 #include <Poco/URI.h>
 
 #include "ClientSession.hpp"
-#include "Common.hpp"
-#include "common/FileUtil.hpp"
-#include "Protocol.hpp"
+#include <Common.hpp>
+#include <Protocol.hpp>
 #include "SenderQueue.hpp"
-#include "Unit.hpp"
-#include "Util.hpp"
+#include <Unit.hpp>
+#include <Util.hpp>
+#include <common/FileUtil.hpp>
+
+using namespace LOOLProtocol;
 
 using Poco::DirectoryIterator;
 using Poco::File;
 using Poco::StringTokenizer;
 using Poco::Timestamp;
 
-using namespace LOOLProtocol;
-
 TileCache::TileCache(const std::string& docURL,
                      const Timestamp& modifiedTime,
                      const std::string& cacheDir) :
@@ -73,6 +72,7 @@ TileCache::TileCache(const std::string& docURL,
 
 TileCache::~TileCache()
 {
+    _owner = std::thread::id(0);
     LOG_INF("~TileCache dtor for uri [" << _docURL << "].");
 }
 
@@ -114,7 +114,7 @@ std::shared_ptr<TileCache::TileBeingRendered> TileCache::findTileBeingRendered(c
 {
     const std::string cachedName = cacheFileName(tileDesc);
 
-    Util::assertIsLocked(_tilesBeingRenderedMutex);
+    assertCorrectThread();
 
     const auto tile = _tilesBeingRendered.find(cachedName);
     return tile != _tilesBeingRendered.end() ? tile->second : nullptr;
@@ -124,7 +124,7 @@ void TileCache::forgetTileBeingRendered(const TileDesc& tile)
 {
     const std::string cachedName = cacheFileName(tile);
 
-    Util::assertIsLocked(_tilesBeingRenderedMutex);
+    assertCorrectThread();
 
     _tilesBeingRendered.erase(cachedName);
 }
@@ -149,12 +149,12 @@ std::unique_ptr<std::fstream> TileCache::lookupTile(const TileDesc& tile)
 
 void TileCache::saveTileAndNotify(const TileDesc& tile, const char *data, const size_t size)
 {
-    std::unique_lock<std::mutex> lock(_tilesBeingRenderedMutex);
+    assertCorrectThread();
 
     std::shared_ptr<TileBeingRendered> tileBeingRendered = findTileBeingRendered(tile);
 
     // Save to disk.
-    const auto cachedName = (tileBeingRendered ? tileBeingRendered->getCacheName()
+    const std::string cachedName = (tileBeingRendered ? tileBeingRendered->getCacheName()
                                                : cacheFileName(tile));
 
     // Ignore if we can't save the tile, things will work anyway, but slower.
@@ -168,7 +168,7 @@ void TileCache::saveTileAndNotify(const TileDesc& tile, const char *data, const
     // Notify subscribers, if any.
     if (tileBeingRendered)
     {
-        const auto subscriberCount = tileBeingRendered->_subscribers.size();
+        const size_t subscriberCount = tileBeingRendered->_subscribers.size();
         if (subscriberCount > 0)
         {
             std::string response = tile.serialize("tile:");
@@ -182,7 +182,7 @@ void TileCache::saveTileAndNotify(const TileDesc& tile, const char *data, const
             payload->append(data, size);
 
             auto& firstSubscriber = tileBeingRendered->_subscribers[0];
-            auto firstSession = firstSubscriber.lock();
+            std::shared_ptr<ClientSession> firstSession = firstSubscriber.lock();
             if (firstSession)
                 firstSession->enqueueSendMessage(payload);
 
@@ -201,7 +201,7 @@ void TileCache::saveTileAndNotify(const TileDesc& tile, const char *data, const
                 for (size_t i = 1; i < subscriberCount; ++i)
                 {
                     auto& subscriber = tileBeingRendered->_subscribers[i];
-                    auto session = subscriber.lock();
+                    std::shared_ptr<ClientSession> session = subscriber.lock();
                     if (session)
                     {
                         session->enqueueSendMessage(payload);
@@ -321,8 +321,7 @@ void TileCache::invalidateTiles(int part, int x, int y, int width, int height)
 
     File dir(_cacheDir);
 
-    std::unique_lock<std::mutex> lock(_cacheMutex);
-    std::unique_lock<std::mutex> lockSubscribers(_tilesBeingRenderedMutex);
+    assertCorrectThread();
 
     if (dir.exists() && dir.isDirectory())
     {
@@ -444,9 +443,9 @@ void TileCache::subscribeToTileRendering(const TileDesc& tile, const std::shared
 {
     std::ostringstream oss;
     oss << '(' << tile.getPart() << ',' << tile.getTilePosX() << ',' << tile.getTilePosY() << ')';
-    const auto name = oss.str();
+    const std::string name = oss.str();
 
-    std::unique_lock<std::mutex> lock(_tilesBeingRenderedMutex);
+    assertCorrectThread();
 
     std::shared_ptr<TileBeingRendered> tileBeingRendered = findTileBeingRendered(tile);
 
@@ -493,9 +492,9 @@ std::string TileCache::cancelTiles(const std::shared_ptr<ClientSession> &subscri
     assert(subscriber && "cancelTiles expects valid subscriber");
     LOG_TRC("Cancelling tiles for " << subscriber->getName());
 
-    std::unique_lock<std::mutex> lock(_tilesBeingRenderedMutex);
+    assertCorrectThread();
 
-    const auto sub = subscriber.get();
+    const ClientSession* sub = subscriber.get();
 
     std::ostringstream oss;
 
@@ -530,8 +529,18 @@ std::string TileCache::cancelTiles(const std::shared_ptr<ClientSession> &subscri
         ++it;
     }
 
-    const auto canceltiles = oss.str();
+    const std::string canceltiles = oss.str();
     return canceltiles.empty() ? canceltiles : "canceltiles " + canceltiles;
 }
 
+void TileCache::assertCorrectThread()
+{
+    const bool correctThread = _owner == std::thread::id(0) || std::this_thread::get_id() == _owner;
+    if (!correctThread)
+        LOG_ERR("TileCache method invoked from foreign thread. Expected: " <<
+                Log::to_string(_owner) << " but called from " <<
+                std::this_thread::get_id() << " (" << Util::getThreadId() << ").");
+    assert (correctThread);
+}
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
